#include <vector>
#include <iostream>
#include <string>
#include <stdint.h>

class Metadata {
public:
	Metadata(const std::string& t, unsigned long s, unsigned long l) :
	     type(t), type_size(s), length(l) {};
	virtual ~Metadata() {};
	inline std::string get_type(void) const { return type; };
	inline unsigned long get_type_size(void) const { return type_size; };
	inline unsigned long get_len(void) const { return length; };
	inline unsigned long get_tot_size(void) const 
						{ return length * type_size; };
protected:
	std::string type;
	unsigned long type_size;
	unsigned long length;
};

class Data : public Metadata {
public:
	Data(const std::string& t, const std::string& n, 
	     unsigned long s, unsigned long l) :
	     Metadata(t, s, l), name(n) {};
	inline std::string get_name(void) const { return name; };
protected:
	std::string name;
};

template<class T>
class DataTemplate : public Data {
public:
	DataTemplate(const std::string& type, const std::string& name, 
	             unsigned long len) : 
	             Data(type, name, sizeof(T), len)
	{
		std::cout << __PRETTY_FUNCTION__ << ": name=" << name << 
		             std::endl;
		actual_data = new T[len];
	};
	~DataTemplate()
	{
		std::cout << __PRETTY_FUNCTION__ << ": name=" << get_name() << 
		        std::endl;
		delete[] actual_data;
	};
	
	T* actual_data;
};

class DataInt8 : public DataTemplate<int8_t> {
public:
	DataInt8(const std::string& name, unsigned long len) : 
	    DataTemplate<int8_t>("int8", name, len) {};
};

class DataUInt16 : public DataTemplate<uint16_t> {
public:
	DataUInt16(const std::string& name, unsigned long len) : 
	    DataTemplate<uint16_t>("uint16", name, len) {};
};

class DataFloat32 : public DataTemplate<float> {
public:
	DataFloat32(const std::string& name, unsigned long len) : 
	    DataTemplate<float>("float32", name, len) {};
};

using namespace std;

struct XmlDataElement {
	std::string type;
	std::string name;
	unsigned len;
};

std::vector<XmlDataElement> parse_xml_data(/*might have params..*/) 
{
	// creates the output vector...
	vector<XmlDataElement> out;
	XmlDataElement e1, e2, e3, e4;

	e1.type = "uint16";
	e1.name = "data_name_1";
	e1.len = 2;

	e2.type = "float32";
	e2.name = "data_name_2";
	e2.len = 1;

	e3.type = "int8";
	e3.name = "data_name_3";
	e3.len = 4;

	e4.type = "bad_data_type";
	e4.name = "dontcare";
	e4.len = 1;

	out.push_back(e1);
	out.push_back(e2);
	out.push_back(e3);
	out.push_back(e4);

	return out;
}



class FormatHandler {
public:
	FormatHandler();
	~FormatHandler();
	void format(uint8_t* rawdata, unsigned long len);
private:
	std::vector<Data*> data_vector;
};


FormatHandler::FormatHandler()
{
	// We create the data vector according to XML config
	vector<XmlDataElement> config = parse_xml_data();

	vector<XmlDataElement>::const_iterator it;
	for (it = config.begin(); it != config.end(); ++it) {
		Data *d;
		if (it->type == "int8") 
			 d = new DataInt8(it->name, it->len);
		else if (it->type == "uint16") 
			 d = new DataUInt16(it->name, it->len);
		else if (it->type == "float32") 
			 d = new DataFloat32(it->name, it->len);
		/*
		else {
			cout << "Unknown data type = '" << it->type << 
			        "' ! Skipping" << endl;
			continue;
		}
		*/
		data_vector.push_back(d);
	}
}


FormatHandler::~FormatHandler()
{
	// We have to destroy the data vector properly
	vector<Data*>::iterator it;
	for (it = data_vector.begin(); it != data_vector.end(); ++it)
		delete *it;
}

void FormatHandler::format(uint8_t* rawdata, unsigned long len)
{
	// For now we do nothing...
}



int main(void)
{
	{
	FormatHandler formatter;
	} // ~Formatter() is called here!

	return 0;
}
